package Logging;

use strict;
use POSIX;

use vars qw|$verbosity|;
$verbosity = 5;


my %handlerProc;


sub setVerbosity {
  $verbosity = shift;
}

sub setOnError {
 $handlerProc{onerror} = shift;
}

sub setOnWarning {
 $handlerProc{onwarning} = shift;
}

my $outputHandle;

sub setOutput {
  my ($fh) = @_;
  $outputHandle = $fh;
}

sub getOutput{
  return $outputHandle if defined $outputHandle;
  return \*STDERR;
}
sub date {
  return POSIX::strftime("%T", gmtime(time()));
}

sub error {
  getOutput()->print( date(), " ERROR ", @_, "\n" ) if $verbosity > 0;
  $handlerProc{onerror}->( @_ ) if exists $handlerProc{onerror} and ref $handlerProc{onerror} eq 'CODE';
}

sub warning {
  getOutput()->print( date(), " WARN  ", @_, "\n" ) if $verbosity > 1;
  $handlerProc{onwarning}->( @_ ) if exists $handlerProc{onwarning} and ref $handlerProc{onwarning} eq 'CODE';
}

sub info {
  getOutput()->print( date(), " INFO  ", @_, "\n" ) if $verbosity > 2;
}

sub debug {
  getOutput()->print( date(), " DEBUG ", @_, "\n" ) if $verbosity > 3;
}

sub trace {
  getOutput()->print( date(), " TRACE ", @_, "\n" ) if $verbosity > 4;
}

1;