package XmlNode;

#
# ::new($name)
#
# ->setAttribute($name, $value[, $encoding])
# ->setText($text)
# ->addChild($child)
#
# ->serialize(*FH[, $prettyPrint[, $omitXmlDeclaration]])
#
# ->getText() - !!USED ONLY IN CONFIXX AGENT. DO NOT USE IT IN OTHER PLACES. IT IS LAW.!!
#
use strict;

my $startSavePath = '';
my $nocompatibilityProcs;

sub new {
  my $self = {};
  bless( $self, shift );
  $self->_init(@_);
  return $self;
}

sub setStartSavePath{
  my ( $path ) = @_;
  $startSavePath = $path;
}

sub resetCompatibilityProcs{
  $nocompatibilityProcs = 1;
}

sub ReleaseCode {
  my $self = shift;
  undef $self->{'ATTRIBUTE'};
  undef $self->{'CONTENT'};
  undef $self->{'ADDCHILD'};
  undef $self->{'PRINT_TREE'};
  undef $self->{'PRINT'};

  my $child;
  foreach $child ( @{ $self->{children} } ) {
    $child->ReleaseCode();
  }
}

sub _init {
  my ( $self, $name, %args ) = @_;

  $self->{name}       = $name;
  $self->{attributes} = {};
  $self->{children}   = [];
  $self->{text}       = undef;

  my ( $option, $value );
  while ( ( $option, $value ) = each %args ) {
    if ( $option eq "content" ) {
      if ( defined $value ) {
        $self->setText($value);
      }
    }
    elsif ( $option eq "attributes" ) {
      my %attributes = %{$value};
      my ( $attrname, $attrvalue );
      while ( ( $attrname, $attrvalue ) = each %attributes ) {
        $self->setAttribute( $attrname, $attrvalue );
      }
    }
    elsif ( $option eq "children" ) {
      $self->addChild($_) foreach @{$value};
    }
    else {
      die "Unknown option passed to XmlNode: $option";
    }
  }

  #COMPATIBILITY
  if( not $nocompatibilityProcs ){
    $self->{'ATTRIBUTE'} = sub { return $self->setAttribute(@_); };
    $self->{'CONTENT'} =
      sub { (@_) ? return $self->setText(@_) : return $self->getText(); };
    $self->{'ADDCHILD'}   = sub { return $self->addChild(@_); };
    $self->{'PRINT_TREE'} = sub { if( scalar(@_)==2 ) { return $self->serializeChild( $_[0], $_[1] ); } else { return $self->serialize( $_[0] ); } };
    $self->{'PRINT'}      = sub { return $self->serialize( $_[0], 0, 1 ) };
  }
}

sub setAttribute {
  my ( $self, $name, $value, $encoding ) = @_;
  die "'undef' name passed to XmlNode::addAttribute" if !$name;
  die "'undef' value passed to XmlNode::addAttribute for '$name'" if !defined $value;

  $value = _xmlAttributeEscape( Encoding::encode( $value, $encoding ) );
  $self->{attributes}->{$name} = $value;
}

sub getAttribute {
  my ( $self, $name ) = @_;
  die "'undef' name passed to XmlNode::addAttribute" if !$name;

  my $ret;
  while ((my $attrName, my $value) = each %{$self->{attributes}}) {
    if ($name eq $attrName) {
      $ret = $value;
    }
  }

  return $ret;
}

sub setText {
  my ( $self, $text ) = @_;
  $self->{text} = _xmlTextEscape( Encoding::encode($text) );
  $self->_sanityCheck();
}

sub getText {
  my ($self) = @_;
  return $self->{text};
}

sub addChild {
  my ( $self, $childNode, $position ) = @_;
  if( $position ){
     unshift @{ $self->{children} }, $childNode;
  }
  else{
     push @{ $self->{children} }, $childNode;
  }
  $self->_sanityCheck();
}

sub getChildren {
  my ( $self, $childName ) = @_;

  my @ret;

  foreach my $child (@{$self->{children}}) {
    push @ret, $child if ($child->{name} eq $childName);
  }

  return @ret;
}

sub getChild {
  my ( $self, $childName, $create, $position ) = @_;

  my $ret;
  foreach my $child (@{$self->{children}}) {
    if( $child and $child->{name} eq $childName) {
	    die "The element '$self->{name}' has multiply elements '$childName', expected one!" if $ret;
	    $ret = $child;
    }
  }
  if( not $ret and $create ){
    $ret = XmlNode->new( $childName );
    $self->addChild( $ret, $position );
  }
  return $ret;
}

sub serialize {
  my ( $self, $fh, $prettyPrint, $omitXmlDeclaration ) = @_;

  if ( !$omitXmlDeclaration ) {
    print $fh '<?xml version="1.0" encoding="UTF-8"?>';
    print $fh "\n";
  }

  $self->_serializeNode( $fh, 1, $prettyPrint );
}

sub serializeChild {
  my ( $self, $fh, $child ) = @_;

  print $fh '<?xml version="1.0" encoding="UTF-8"?>';
  print $fh "\n";

  print $fh $self->_serializeTag( 1, 0 );
  my $dumpInfo = $self->getChild( 'dump-info' );
  $dumpInfo->_serializeNode( $fh, 1, 1, ) if $dumpInfo;
  $child->_serializeNode( $fh, 1, 1, );
  print $fh $self->_serializeTag( 0, 1 );
}

sub _serializeNode {
  my ( $self, $fh, $indent, $prettyPrint ) = @_;

  print $fh "  " x $indent;

  if ( !@{ $self->{children} } && !defined $self->{text} ) {
    print $fh $self->_serializeTag( 1, 1 );
    return;
  }

  print $fh $self->_serializeTag( 1, 0 );

  my $child;
  foreach $child ( @{ $self->{children} } ) {
    $child->_serializeNode( $fh, $indent + 1, $prettyPrint ) if defined $child;
  }
  print $fh ( defined $self->{text} ) ? $self->{text} : "  " x $indent;

  print $fh $self->_serializeTag( 0, 1 );
}

sub _serializeTag {
  my ( $self, $start, $stop ) = @_;

  my $out;
  if ($start) {
    $out = "<" . $self->{name};
    my ( $key, $value );
    while ( ( $key, $value ) = each( %{ $self->{attributes} } ) ) {
      if( $startSavePath and $self->{name} eq 'cid' and $key eq 'path' ){
        $value = substr( $value, length($startSavePath)+1 ) if index( $value, $startSavePath )==0;
      }
      $out .= " $key=\"$value\"";
    }
    if ($stop) {
      $out .= "/>\n";
    }
    else {
      $out .= ( defined $self->{text} ) ? ">" : ">\n";
    }
  }
  else {
    $out = "</" . $self->{name} . ">\n";
  }
  return $out;
}

sub _sanityCheck {
  my ($self) = @_;
  if ( defined $self->{text} and @{ $self->{children} } ) {
    die "Both text and children nodes specified for <$self->{name}/>";
  }
}

sub getContentSize{
 my ($self) = @_;
 my $size = 0;
 foreach my $child ( @{$self->{children} } ) {
    if( $child ) {
      if( $child->{name} eq 'cid' ){
        if( not exists $child->{attributes}->{'referrer'} ) {
          foreach my $cid( @{$child->{children}} ){
            $size += $cid->{attributes}->{'size'};
          }
        }
      }
      elsif( scalar( @{$child->{children}} ) > 0 ) {
        $size += $child->getContentSize();
      }
    }
 }
 return $size;
}

sub getGuid{
  my ($self) = @_;
  return $self->{attributes}->{'guid'}
}

# -- utilities --

sub _xmlTextEscape {
  my ($text) = @_;
  if ( $text =~ /[&<'"]/s ) {
    $text =~ s/&/&amp;/sg;
    $text =~ s/</&lt;/sg;
    $text =~ s/\'/&#39;/sg;
    $text =~ s/\"/&quot;/sg;
  }
  return $text;
}

# Normalization according to the XML 1.0 (Third Edition),
# clauses 2.3 "Common Syntactic Constructs"
# and 3.3.3 "Attribute-value normalization"

sub _xmlAttributeEscape {
  my ($text) = @_;

  if ( $text =~ /[&\r\n\t<"']/s ) {
    $text =~ s/\&/&amp;/sg;
    $text =~ s/\r/&#xD;/sg;
    $text =~ s/\n/&#xA;/sg;
    $text =~ s/\t/&#x9;/sg;
    $text =~ s/\</&lt;/sg;
    $text =~ s/\"/&quot;/sg;
    $text =~ s/\'/&#39;/sg;
  }
  return $text;
}

1;

# Local Variables:
# mode: cperl
# cperl-indent-level: 2
# indent-tabs-mode: nil
# tab-width: 4
# End: